import unittest
import Zope

from DateTime import DateTime
from Products.ParsedXML.ParsedXML import ParsedXML

from Products.Silva.tests.test_SilvaObject import hack_create_user
from Testing import makerequest

from Products.SilvaReferenceChecker.ReferenceChecker import ReferenceInfo
import Products.SilvaReferenceChecker.monkeys_silva

class ReferenceTestCase(unittest.TestCase):
    """ test that the reference aggregation does actually collect
    the right references.
    the tests are only written for documents and "special content"
    objects, as for the "normal content" object (i,.e. with parsed xml
    versions) the content check should be good enough.
    """

    def assertEqualElements(self, seq1, seq2):
        """ asserts that both lists contain the same entries
        no matter in which order
        """
        for x in seq1:
            if x not in seq2:
                raise AssertionError(
'''%s not in sequence.
 expected sequence:\n\t%s
 got sequence:\n\t%s''' %\
                                     (x, '\n\t'.join(map(str,seq1)), \
                                      '\n\t'.join(map(str,seq2))) )
        for y in seq2:
            if y not in seq1:
                raise AssertionError(
'''%s too much in sequence.
 expected sequence:\n\t%s
 got sequence:\n\t%s''' % \
                                     (y, '\n\t'.join(map(str,seq1)),
                                         '\n\t'.join(map(str,seq2)) ))


        
    def setUp(self):

        get_transaction().begin()
        self.connection = Zope.DB.open()
        try:
            self.root = makerequest.makerequest(
                self.connection.root()['Application'])
            # to fix problem with the permission check
            self.root.REQUEST['URL1'] = '' 
            hack_create_user(self.root)
                
            self.root.manage_addProduct['Silva'].\
                      manage_addRoot('root', 'Silva Root')
            self.sroot = self.root.root

            # Doc, Folder
            self.sroot.manage_addProduct['Silva'].manage_addDocument(
                'test_doc1','Test doc')
            self.doc1 = self.sroot.test_doc1

            addFolder = self.sroot.manage_addProduct['Silva'].manage_addFolder
            addFolder('test_folder', 'Test Folder')
            self.folder = self.sroot.test_folder

            # Ghost
            add_ghost = self.folder.manage_addProduct['Silva'].manage_addGhost
            add_ghost('test_ghost1', '/test_folder/test_doc2')
            self.ghost = self.folder.test_ghost1

        except:
            self.tearDown()
            raise

    def tearDown(self):
        get_transaction().abort()
        self.connection.close()

    def _publish(self, obj):
        assertTrue = self.assert_
        if obj.implements_container():
            obj = obj.get_default()
        assertTrue(obj.get_editable())
        obj.set_unapproved_version_publication_datetime(DateTime()-1)
        obj.approve_version()
        assertTrue(obj.get_viewable())
        


    def test_Document(self):
        """ test links in a document """

        # This document should contain the following references:
        # - a normal internal link in text,
        # - an external link,
        # - an image (with surrounding link, count as two references)
        # - a code element
        # Total: 5
        # XXX: maybe test for links in nested structures, e.g. table cells?

        xml = """\
        <doc>
        <p>This is a link <link url="/test_doc2">Ok</link> here</p>
        <p><link url="http://zope.org">External link</link></p>
        <image link="/image_link" path="/root/test_image" />
        <code path="/root/code_source/some_script" />
        </doc>
        """
        self.doc1.get_editable().content.manage_edit(xml)
        self._publish(self.doc1)

        expected = (
            ReferenceInfo(self.doc1, '/test_doc2'),
            ReferenceInfo(self.doc1, 'http://zope.org'),
            ReferenceInfo(self.doc1, '/image_link'),
            ReferenceInfo(self.doc1, '/root/test_image',
                          path_info=ReferenceInfo.PATH_ZOPE_ROOT,
                          ref_type=ReferenceInfo.REF_INTERN,
                          restriction=ReferenceInfo.RESTRICT_IMAGE),
            ReferenceInfo(self.doc1, '/root/code_source/some_script',
                          path_info=ReferenceInfo.PATH_ZOPE_ROOT,
                          ref_type=ReferenceInfo.REF_INTERN,
                          restriction=ReferenceInfo.RESTRICT_CODE),
            )
        references = self.doc1.get_references()
        
        #print references
        #print expected
        #self.assertEquals(str(expected), str(references))
        self.assertEqualElements(expected, references)

        # do the same for empty references;
        # they should be aggregated as well here
        xml = """\
        <doc>
        <p>This is a link <link url="">Ok</link> here</p>
        <p><link url="">Second link</link></p>
        <image></image>
        <code></code>
        </doc>
        """
        
        # ugh, hack out permission problem
        self.doc1._verifyObjectPaste = lambda x: 1
        self.doc1.create_copy()
        self.doc1.get_editable().content.manage_edit(xml)
        self._publish(self.doc1)

        expected = (
            ReferenceInfo(self.doc1, '',
                          ref_type=ReferenceInfo.REF_UNKNOWN),
            ReferenceInfo(self.doc1, '',
                          ref_type=ReferenceInfo.REF_UNKNOWN),
            ReferenceInfo(self.doc1, '',
                          path_info=ReferenceInfo.PATH_CONTAINER,
                          ref_type=ReferenceInfo.REF_INTERN,
                          restriction=ReferenceInfo.RESTRICT_IMAGE),
            ReferenceInfo(self.doc1, '',
                          path_info=ReferenceInfo.PATH_ZOPE_ROOT,
                          ref_type=ReferenceInfo.REF_INTERN,
                          restriction=ReferenceInfo.RESTRICT_CODE),
            )
        references = self.doc1.get_references()

        self.assertEqualElements(expected, references)




    def test_ghost(self):
        references = self.ghost.get_references()
        self.assertEqualElements((), references)
        
        self._publish(self.ghost)
        expected =( ReferenceInfo(self.ghost,'/test_folder/test_doc2',
                                  restriction=ReferenceInfo.RESTRICT_CONTENT,
                                  ref_type=ReferenceInfo.REF_INTERN,
                                  path_info=ReferenceInfo.PATH_ZOPE_ROOT) ,)
        references = self.ghost.get_references()
        self.assertEqualElements(expected, references)
        


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ReferenceTestCase, 'test'))
    return suite        

def main():
    unittest.TextTestRunner(verbosity=1).run(test_suite())

if __name__ == '__main__':
    main()
